#!/usr/bin/env python3
#### # -*- encoding=utf8 -*-

import codecs
import csv
import io
import json
import os
import sys

def eprint(*args, **kwargs):
    print(*args, file=sys.stderr, **kwargs)

def checkUserName(username):
    return -1 == username.find('/') and -1 == username.find(':') and -1 == username.find(',')

def main():

    if len(sys.argv) != 2:
        eprint(f"usage: {sys.argv[0]} <csv file>")
        exit(1)

    csv_file_path = sys.argv[1]
    if not os.path.isfile(csv_file_path):
        eprint(f'file not found [{csv_file_path}]')
        exit(1)

    user_info_list = list()
    remote_name_set = set()
    local_name_set = set()

    bytes = min(32, os.path.getsize(csv_file_path))
    raw = open(csv_file_path, 'rb').read(bytes)
    if raw.startswith(codecs.BOM_UTF8):
        encoding = 'utf-8-sig'
    else:
        encoding = 'utf8'

    with io.open(csv_file_path, 'r', encoding=encoding) as fp:
        csv_reader = csv.reader(fp, csv.excel)
        for row in csv_reader:
            try:
                typ, remote_name, local_name = row

                if typ not in ('user', 'user_mapping', 'group'):
                    raise ValueError
                #duplicate record is not allowed
                if remote_name in remote_name_set or local_name in local_name_set:
                    raise ValueError

                if not (checkUserName(remote_name) and checkUserName(local_name)):
                    continue

                remote_name_set.add(remote_name)
                local_name_set.add(local_name)

                user_info = {'type': typ, 'remote_name': remote_name, 'local_name': local_name}
                user_info_list.append(user_info)
            except ValueError as e:
                #FIXME log error
                eprint('parse error for "{0}"'.format(row))
                continue

    json.dump(user_info_list, sys.stdout)

if __name__ == '__main__':
    main()
